#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "movement.h"

void MainWindow::servoChange(QString oldId, servoFrame* serv)
{
    if(declineServoPos==1)return;
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL){
        return;
    }else if(item->parent()==NULL){
        return;
    };
    Position* p=(Position*)item;
    int index=-1;
    for(int i=0; i<p->count(); i++)if(p->item(i)->id()==serv->id())index=i;
    if(serv->selected()){
        if(index>=0){
            ServoPos* sp=p->item(index);
            sp->setPos(serv->pos());
        }else{
            p->add(serv->id(),serv->pos());
        };
    }else{
        if(index>=0)p->remove(index);
    };
}

void MainWindow::updateServoPosOrDisable()
{
    int disable=0;
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL){
        disable=1;
    }else if(item->parent()==NULL){
        disable=1;
    };
    if(disable==1){
        for(int i=0; i<servoQty; i++)servo[i]->setEnabled(false);
    }else{
        declineServoPos=1;
        Position* p=(Position*)item;
        for(int i=0; i<servoQty; i++)
        {
            servo[i]->setEnabled(true);
            servo[i]->setSelected(false);
            servo[i]->setPos(0);
            for(int j=0; j<p->count(); j++)
            {
                ServoPos* sp=p->item(j);
                if(sp->id()==servo[i]->id()){
                    servo[i]->setPos(sp->pos());
                    servo[i]->setSelected(true);
                };
            };
        };
        declineServoPos=0;
    };
}

void MainWindow::addServo()
{
    servoFrame* s=new servoFrame(this);
    servo[servoQty]=s;
    s->setGeometry(200,200,120,160);
    s->show();
    servoQty++;
    connect(s,SIGNAL(onServoChange(QString,servoFrame*)),this,SLOT(servoChange(QString,servoFrame*)));
}

void MainWindow::updateTitle()
{
    this->setWindowTitle("OR-Sequencer v0.10 - "+filename+(modifiedFlag?" *":""));
}

void MainWindow::removeServo(int n)
{
    if(n>=servoQty)return;
    servo[n]->hide();
    delete(servo[n]);
    servoQty--;
    for(int i=n; i<servoQty; i++)servo[i]=servo[i+1];
}

void MainWindow::fileNew()
{
    if(modifiedFlag==1)
    {
        //Offer user to save current project
    };
    while(servoQty>0)removeServo(0);
    filename="";
    modifiedFlag=1;
    updateTitle();
}

void MainWindow::fileOpen()
{
    declineServoPos=1;
    //read file
    declineServoPos=0;
    this->updateServoPosOrDisable();
}

void MainWindow::fileSave()
{
}

void MainWindow::fileSaveAs()
{
}


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    servoQty=0;
    moveFlag=0;
    actionFlag=0;
    declineServoPos=0;
    filename="";
    modifiedFlag=1;
    updateTitle();
    connect(ui->actionExit,SIGNAL(triggered()),this,SLOT(close()));
    connect(ui->actionAdd_servo,SIGNAL(triggered()),this,SLOT(addServo()));
    connect(ui->actionNew,SIGNAL(triggered()),this,SLOT(fileNew()));
    connect(ui->actionOpen,SIGNAL(triggered()),this,SLOT(fileOpen()));
    connect(ui->actionSave,SIGNAL(triggered()),this,SLOT(fileSave()));
    connect(ui->actionSave_as,SIGNAL(triggered()),this,SLOT(fileSaveAs()));

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::mouseMoveEvent(QMouseEvent* event)
{
    if( !(event->buttons() & Qt::LeftButton) )return;
    if( (moveFlag==1) || ((event->pos()-startDrag).manhattanLength()>=QApplication::startDragDistance()) )
    {
        QRect r;
        for(int i=0; i<servoQty; i++)
        if(servo[i]->frameShadow()==QFrame::Sunken)
        {
            r=servo[i]->geometry();
            r.setTopLeft(event->pos()-startDrag+r.topLeft());
            r.setWidth(120);
            r.setHeight(160);
            servo[i]->setGeometry(r);
        }
        moveFlag=1;
        startDrag=event->pos();
    }
}

void MainWindow::mousePressEvent(QMouseEvent* event)
{

    if(event->button() != Qt::LeftButton)return;
    int sel=-1;
    for(int i=0; i<servoQty; i++)if(servo[i]->geometry().contains(event->pos()))sel=i;
    if(event->modifiers() & Qt::AltModifier){
        if(sel>=0) servo[sel]->setFrameShadow(QFrame::Raised);
        actionFlag=1;
    }else{
        if( !(event->modifiers() & Qt::ShiftModifier) )
            for(int i=0; i<servoQty; i++)servo[i]->setFrameShadow(QFrame::Raised);
        if(sel>=0){
            servo[sel]->setFrameShadow(QFrame::Sunken);
            actionFlag=1;
        };
    };
    startDrag=event->pos();
}

void MainWindow::mouseReleaseEvent(QMouseEvent* event)
{
    if(actionFlag==0)for(int i=0; i<servoQty; i++)servo[i]->setFrameShadow(QFrame::Raised);
    if(moveFlag==1){
        int selQty=0;
        for(int i=0; i<servoQty; i++)if(servo[i]->frameShadow()==QFrame::Raised)selQty++;
        if(selQty==1)for(int i=0; i<servoQty; i++)servo[i]->setFrameShadow(QFrame::Raised);
    };
    actionFlag=0;
    moveFlag=0;
}

void MainWindow::on_treeWidget_itemSelectionChanged()
{
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL){
        ui->delMoveBtn->setEnabled(false);
        ui->addPosBtn->setEnabled(false);
        ui->delPosBtn->setEnabled(false);
    }else if(item->parent()==NULL){
        ui->delMoveBtn->setEnabled(true);
        ui->addPosBtn->setEnabled(true);
        ui->delPosBtn->setEnabled(false);
    }else{
        ui->delMoveBtn->setEnabled(true);
        ui->addPosBtn->setEnabled(true);
        ui->delPosBtn->setEnabled(true);
    };
    updateServoPosOrDisable();
}

void MainWindow::on_addMoveBtn_clicked()
{
    Movement* m=new Movement();
    m->setText(0,"new movement");
    m->setText(1,"0");
    ui->treeWidget->addTopLevelItem(m);
    ui->treeWidget->expandItem(m);
}

void MainWindow::on_delMoveBtn_clicked()
{
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL)return;
    while(item->parent()!=NULL)item=item->parent();
    Movement* m=(Movement*)item;
    ui->treeWidget->invisibleRootItem()->removeChild(m);
    delete(m);
}

void MainWindow::on_addPosBtn_clicked()
{
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL)return;
    while(item->parent()!=NULL)item=item->parent();
    Movement* m=(Movement*)item;
    Position* p=new Position();
    p->setText(0,"new position");
    p->setText(1,"10");
    m->addChild(p);
    m->updateTiming();
    ui->treeWidget->expandItem(m);
}

void MainWindow::on_delPosBtn_clicked()
{
    QTreeWidgetItem* item=ui->treeWidget->currentItem();
    if(item==NULL)return;
    if(item->parent()==NULL)return;
    Position* p=(Position*)item;
    Movement* m=(Movement*)p->parent();
    m->removeChild(p);
    m->updateTiming();
    delete(p);
}
